from urllib.error import HTTPError

import pytest
import requests
from rdflib import Graph
from rdflib.plugins.stores.sparqlstore import SPARQLStore, SPARQLUpdateStore


def add_data(update_endpoint):
    store = SPARQLUpdateStore(update_endpoint=update_endpoint)
    store.update(SPARQL_UPDATE_QUERY)


def is_http_error_code(code):
    return 400 <= code < 500


SPARQL_SPO_QUERY = """
    SELECT ?s ?p ?o
    WHERE {
        ?s ?p ?o .
    }
    LIMIT 1
"""

SPARQL_UPDATE_QUERY = """
    PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
    INSERT DATA {
        <http://example.com/book/42> rdfs:label "Hitchhiker's Guide" .
    }
"""

PAYLOAD_TTL = """
    @prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .
    <http://example.com/book/42> rdfs:label "Hitchhiker's Guide" .
"""


def can_use_sparql_query(fuseki_url, dataset_name, endpoint_name):
    endpoint = f"{fuseki_url}/{dataset_name}/{endpoint_name}"
    store = SPARQLStore(query_endpoint=endpoint)

    result = store.query(SPARQL_SPO_QUERY)
    return len(result) == 1


def can_use_sparql_update(fuseki_url, dataset_name, endpoint_name):
    update_endpoint = f"{fuseki_url}/{dataset_name}/{endpoint_name}"
    read_endpoint = f"{fuseki_url}/{dataset_name}/"
    store = SPARQLUpdateStore(query_endpoint=read_endpoint, update_endpoint=update_endpoint)

    store.update(SPARQL_UPDATE_QUERY)

    assert len(store.query(SPARQL_SPO_QUERY)) == 1
    return True


def cannot_use_sparql_update(fuseki_url, dataset_name, endpoint_name):
    endpoint = f"{fuseki_url}/{dataset_name}/{endpoint_name}"
    store = SPARQLUpdateStore(update_endpoint=endpoint)

    with pytest.raises(HTTPError) as exc_info:
        store.update(SPARQL_UPDATE_QUERY)
    assert is_http_error_code(exc_info.value.status)
    return True


def can_use_gsp_read(fuseki_url, dataset_name, endpoint_name):
    endpoint = f"{fuseki_url}/{dataset_name}/{endpoint_name}?default"
    headers = {"Accept": "text/turtle"}

    r = requests.get(endpoint, headers=headers)
    assert r.status_code == 200
    assert r.headers["content-type"] == "text/turtle"
    assert len(r.content) > 0

    g = Graph().parse(data=r.content)
    assert len(g) > 0
    return True


def can_use_gsp_write(fuseki_url, dataset_name, endpoint_name):
    endpoint = f"{fuseki_url}/{dataset_name}/{endpoint_name}?default"
    headers = {"Content-Type": "text/turtle"}

    r = requests.post(endpoint, data=PAYLOAD_TTL, headers=headers)
    assert r.status_code == 200

    r = requests.put(endpoint, data=PAYLOAD_TTL, headers=headers)
    assert r.status_code == 200

    r = requests.delete(endpoint)
    assert r.status_code == 204

    return True


def cannot_use_gsp_write(fuseki_url, dataset_name, endpoint_name):
    endpoint = f"{fuseki_url}/{dataset_name}/{endpoint_name}?default"
    headers = {"Content-Type": "text/turtle"}

    # RDFLib does not support the Graph Store HTTP Protocol (GSP), thus we use plain HTTP requests.
    r = requests.post(endpoint, data=PAYLOAD_TTL, headers=headers)
    assert is_http_error_code(r.status_code)

    r = requests.put(endpoint, data=PAYLOAD_TTL, headers=headers)
    assert is_http_error_code(r.status_code)

    r = requests.delete(endpoint)
    assert is_http_error_code(r.status_code)

    return True
